package com.jzdsh.common;

import java.net.*;
import java.util.Enumeration;

/**
 * 获取主机ip
 * 
 * @author zsljava
 *
 */
public class InetAddressKit {

	private static volatile String ip = null;

	private InetAddressKit() {
	}

	public static String getIp() {
		if (ip == null) {
			String osName = System.getProperty("os.name").toLowerCase();
			if (osName.indexOf("windows") != -1) {
				ip = IPByWindows();
				return ip;
			}
			ip = IPByLinux();
			return ip;
		}
		return ip;
	}

	public static void main(String[] args) {
		System.out.println(internalIp("192.168.0.72"));
	}

	private synchronized static String IPByWindows() {
		try {
			InetAddress ia = InetAddress.getLocalHost(); // 获得本机网络地址对象
			String hostaddress = ia.getHostAddress(); // 获得对应主机地址
			return hostaddress;
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	private synchronized static String IPByLinux() {
		Enumeration<NetworkInterface> allNetInterfaces; // 定义网络接口枚举类
		String address = null;
		try {
			allNetInterfaces = NetworkInterface.getNetworkInterfaces(); // 获得网络接口
			InetAddress ip = null; // 声明一个InetAddress类型ip地址
			while (allNetInterfaces.hasMoreElements()) // 遍历所有的网络接口
			{
				NetworkInterface netInterface = allNetInterfaces.nextElement();
				System.out.println(netInterface.getName()); // 打印网端名字
				Enumeration<InetAddress> addresses = netInterface.getInetAddresses(); // 同样再定义网络地址枚举类
				while (addresses.hasMoreElements()) {
					ip = addresses.nextElement();
					if (ip != null && (ip instanceof Inet4Address)) // InetAddress类包括Inet4Address和Inet6Address
					{
						System.out.println("Local IP = " + ip.getHostAddress());
						if(!internalIp(ip.getHostAddress())){ // 不是内部ip就直接返回
							return ip.getHostAddress();
						}
						address = ip.getHostAddress();
					}
				}
			}
		} catch (SocketException e) {
			e.printStackTrace();
		}
		return address;
	}

	
	
	 public static boolean internalIp(String ipAddress){/*判断是否是内网IP*/      
	        boolean isInnerIp = false;//默认给定IP不是内网IP      
	        long ipNum = getIpNum(ipAddress);      
	        /** 
	         * 私有IP：A类  10.0.0.0    -10.255.255.255 
	         *       B类  172.16.0.0  -172.31.255.255    
	         *       C类  192.168.0.0 -192.168.255.255   
	         *       D类   127.0.0.0   -127.255.255.255(环回地址)  
	         **/     
	        long aBegin = getIpNum("10.0.0.0");      
	        long aEnd = getIpNum("10.255.255.255");      
	        long bBegin = getIpNum("172.16.0.0");      
	        long bEnd = getIpNum("172.31.255.255");      
	        long cBegin = getIpNum("192.168.0.0");      
	        long cEnd = getIpNum("192.168.255.255");   
	        long dBegin = getIpNum("127.0.0.0");      
	        long dEnd = getIpNum("127.255.255.255");  
	        isInnerIp = isInner(ipNum,aBegin,aEnd) || isInner(ipNum,bBegin,bEnd) || isInner(ipNum,cBegin,cEnd) || isInner(ipNum,dBegin,dEnd);      
	        return isInnerIp;  
	    }  
	       
	    private static long getIpNum(String ipAddress) {/*获取IP数*/      
	        String [] ip = ipAddress.split("\\.");      
	        long a = Integer.parseInt(ip[0]);      
	        long b = Integer.parseInt(ip[1]);      
	        long c = Integer.parseInt(ip[2]);      
	        long d = Integer.parseInt(ip[3]);  
	        long ipNum = a * 256 * 256 * 256 + b * 256 * 256 + c * 256 + d;      
	        return ipNum;      
	    }  
	    
	    private static boolean isInner(long userIp,long begin,long end){      
	        return (userIp>=begin) && (userIp<=end);      
	    }   

}
