package com.jzdsh.common;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.StopWatch;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

public class NumberUtil {
	
	static Random random = new Random();
	/**
	 * 根据最大值与最小值随机获取
	 * @param min
	 * @param max
	 * @return
	 */
	public static int randDom(int min,int max) {
		return random.nextInt(max) % (max - min + 1) + min;
	}
	
	/**
	 * 本地格式化的解析器
	 */
	static NumberFormat numberFormat = NumberFormat.getNumberInstance();
	
	/**
	 * 字符串转金额,可以处理金额 如 5000万,15亿,1.4 亿
	 * @param str
	 * @return
	 * @throws ParseException 
	 */
	public static BigDecimal strToBigDecimal(String str) throws ParseException{
		
		if(StringUtils.isBlank(str)) return null;
		
		str = str.toUpperCase().trim();//统一成大写形式
		
		if(ArrayUtils.contains(new String[]{"非公开","-","--","- -","---"}, str)){
			return null;
		}		
		
		char[] chars = str.toCharArray();

		int coefficient = 1;
		int subIndex = 0;
		for (char c : chars) {
			if(Character.isDigit(c) || ArrayUtils.contains(new char[]{',','.'}, c)){
				subIndex++;
				continue;
			}else{
				switch (c) {
				case '十':
					coefficient *= 10;
					break;
				case '百':
					coefficient *= 100;
					break;
				case '千':
					coefficient *= 1000;
					break;
				case '万':
					coefficient *= 10000;
					break;
				case 'M':
					coefficient *= 1000000;
					break;
				case '亿':
					coefficient *= 100000000;
					break;
				case ' ':
					break;
				default:
					throw new NumberFormatException("Unsupported string conversion!");
				}
			}
		}
		
		return BigDecimal.valueOf(numberFormat.parse(new String(ArrayUtils.subarray(chars, 0, subIndex))).doubleValue()*coefficient);
	}


	/**
	 * 快递数据转换
	 * @param num
	 * @return
	 */
	public static String nFormatterForExpress(long num) {
		List<KeyValuePair<Long,String>> si = new ArrayList<>();
		si.add(new KeyValuePair<Long,String>(1L,""));
		si.add(new KeyValuePair<Long,String>(1000L,"K"));
		si.add(new KeyValuePair<Long,String>(1000000L,"M"));
		si.add(new KeyValuePair<Long,String>(1000000000L,"G"));
		si.add(new KeyValuePair<Long,String>(1000000000000L,"T"));
		int position;
		for (position = si.size()-1; position >0; position--) {
			if (num >= si.get(position).getKey()) {
				break;
			}
		}

		return (num / si.get(position).getKey()) + si.get(position).getValue();
	}
	
	
	/**
	 * 百分比数字转 BigDecimal
	 * @param str
	 * @return
	 */
	public static BigDecimal percentStrToBigDecimal(String str) {
		try {
			NumberFormat nf=NumberFormat.getPercentInstance();
			return BigDecimal.valueOf(nf.parse(str).doubleValue());
		} catch (Exception e) {
			return null;
		}
		
	}

	/**
	 * 	字段符转成DOUBLE，失败后返回NULL
	 * @param str
	 * @return
	 */
	public static Double toDouble(String str) {
		return toDouble(str,null);
	}

	/**
	 * 	字段符转成DOUBLE，失败后返回NULL
	 * @param str
	 * @return
	 */
	public static Double toDouble(final String str, final Double defaultValue) {
		if (StringUtils.isBlank(str)) {
			return defaultValue;
		}
		try {
			return Double.parseDouble(str);
		} catch (final NumberFormatException nfe) {
			return defaultValue;
		}
	}
	
	public static Integer strToInteger(String str) {
		try {
			return Integer.parseInt(str);
		} catch (Exception e) {
			return null;
		}
	}

	final static int [] sizeTable = { 9, 99, 999, 9999, 99999, 999999, 9999999,
			99999999, 999999999, Integer.MAX_VALUE };

	/**
	 * 计算int数字位长
	 * @param x
	 * @return
	 */
	public static int stringSize(int x) {
		x = Math.abs(x);
		for (int i=0; ; i++)
			if (x <= sizeTable[i])
				return i+1;
	}

	/**
	 * 判断方法 判断这个数可以被10除以多少次
	 * @param x
	 * @return
	 */
	public static int stringSizeLong(long x) {
		x = Math.abs(x);
		int count=0;
		while(x>=1) {
			x/=10;
			count++;
		}
		return count;
	}

	
	public static void main(String[] args) throws ParseException {
		//使用系统默认的格式
		DecimalFormat myformat1 = new DecimalFormat("###,###.00");
		String[] strs = {"5000万","15亿","1.4 亿","6 千万","6百 万","1,002,200,999.22","10000","1.2","非公开 "};

		StopWatch sw = new StopWatch();
		sw.start();
		for (int i = 0; i < strs.length; i++) {
				BigDecimal bd = strToBigDecimal(strs[i]);
				System.out.println(bd == null ? null : myformat1.format(bd));
		}
		sw.stop();
		
		System.out.println(sw.toString());
	}
}
